//////////////////////////////////////////////////////////////////////////////
//
// "Embedded C Programming: Techniques and Applications of C and PIC MCUs"
// by Mark Siegusmund
//
// Exercise 18-1 #2
//
// Turns on yellow LED and wait for a button press. After a random delay
// between 5-15 seconds, turns on green LED. Once the green LED is lit the
// user should press the push button again, and the reaction time in
// milliseconds will be displayed.
//
//////////////////////////////////////////////////////////////////////////////

#include <e3.h>

#use timer(timer=1, tick=1ms, bits=32, ISR)

#define SETUP 0
#define WAIT 1
#define RUN 2
#define FINISH 3

int delay;
int mode=SETUP;

#int_ext1
void ext1_isr(void) {
   if(mode==SETUP) {
      mode=WAIT;
      delay=get_timer2()-5;
   }
   else if(mode==RUN) {
      mode=FINISH;
   }
}

void main(void) {
   int32 t;
   
   setup_timer_2(T2_DIV_BY_1, 10, 1);
   enable_interrupts(INT_EXT1_H2L);
   enable_interrupts(GLOBAL);
   
   mode=SETUP;
   delay=0;
   
   for(;;) {
      if(mode==SETUP) {
         output_low(PIN_C3);
         output_low(PIN_C6);
         output_high(PIN_C4);
         set_ticks(0);
      }
      else if(mode==WAIT) {
         if(get_ticks() >= (delay*1000)) {
            mode=RUN;
            output_low(PIN_C3);
            output_low(PIN_C4);
            output_high(PIN_C6);
            set_ticks(0);
         }
      }
      else if(mode==FINISH) {
         mode=SETUP;
         t = get_ticks();
         t *= 1000;
         t /= TICKS_PER_SECOND;
         printf("\r\nIt Took %lu ms", get_ticks());
      }
   }

   for(;;);
}
